@@ -3,10 +3,7 @@ ldif - generate and parse LDIF data (see RFC 2849)
 
 See https://www.python-ldap.org/ for details.
 """
-
-from __future__ import unicode_literals
-
-__version__ = '3.0.0'
+__version__ = '3.4.0'
 
 __all__ = [
   # constants
@@ -25,7 +22,8 @@ from base64 import b64encode, b64decode
 from io import StringIO
 import warnings
 
-from ldap.compat import urlparse, urlopen
+from urllib.parse import urlparse
+from urllib.request import urlopen
 
 attrtype_pattern = r'[\w;.-]+(;[\w_-]+)*'
 attrvalue_pattern = r'(([^,]|\\,)+|".*?")'
@@ -150,8 +148,8 @@ class LDIFWriter:
     entry
         dictionary holding an entry
     """
-    for attr_type in sorted(entry.keys()):
-      for attr_value in entry[attr_type]:
+    for attr_type, values in sorted(entry.items()):
+      for attr_value in values:
         self._unparseAttrTypeandValue(attr_type,attr_value)
 
   def _unparseChangeRecord(self,modlist):
