@@ -24,11 +24,16 @@
   /* openldap.h with ldap_init_fd() was introduced in 2.4.48
    * see https://bugs.openldap.org/show_bug.cgi?id=8671
    */
+#define HAVE_LDAP_INIT_FD 1
 #include <openldap.h>
+#elif (defined(__APPLE__) && (LDAP_VENDOR_VERSION == 20428))
+/* macOS system libldap 2.4.28 does not have ldap_init_fd symbol */
+#undef HAVE_LDAP_INIT_FD
 #else
   /* ldap_init_fd() has been around for a very long time
    * SSSD has been defining the function for a while, so it's probably OK.
    */
+#define HAVE_LDAP_INIT_FD 1
 #define LDAP_PROTO_TCP 1
 #define LDAP_PROTO_UDP 2
 #define LDAP_PROTO_IPC 3
