@@ -3,8 +3,7 @@
 
 #include "common.h"
 #include "constants.h"
-#include "lber.h"
-#include "ldap.h"
+#include "ldapcontrol.h"
 
 /* the base exception class */
 
@@ -32,7 +31,8 @@ static PyObject *errobjects[LDAP_ERROR_MAX - LDAP_ERROR_MIN + 1];
 PyObject *
 LDAPerr(int errnum)
 {
-    if (errnum >= LDAP_ERROR_MIN && errnum <= LDAP_ERROR_MAX) {
+    if (errnum >= LDAP_ERROR_MIN && errnum <= LDAP_ERROR_MAX &&
+            errobjects[errnum + LDAP_ERROR_OFFSET] != NULL) {
         PyErr_SetNone(errobjects[errnum + LDAP_ERROR_OFFSET]);
     }
     else {
@@ -48,38 +48,82 @@ LDAPerr(int errnum)
 
 /* Convert an LDAP error into an informative python exception */
 PyObject *
-LDAPerror(LDAP *l, char *msg)
+LDAPraise_for_message(LDAP *l, LDAPMessage *m)
 {
     if (l == NULL) {
         PyErr_SetFromErrno(LDAPexception_class);
+        ldap_msgfree(m);
         return NULL;
     }
     else {
-        int myerrno, errnum, opt_errnum;
+        int myerrno, errnum, opt_errnum, msgid = -1, msgtype = 0;
         PyObject *errobj;
         PyObject *info;
         PyObject *str;
         PyObject *pyerrno;
-        char *matched, *error;
+        PyObject *pyresult;
+        PyObject *pyctrls = NULL;
+        char *matched = NULL, *error = NULL, **refs = NULL;
+        LDAPControl **serverctrls = NULL;
 
         /* at first save errno for later use before it gets overwritten by another call */
         myerrno = errno;
 
-        opt_errnum = ldap_get_option(l, LDAP_OPT_ERROR_NUMBER, &errnum);
-        if (opt_errnum != LDAP_OPT_SUCCESS)
-            errnum = opt_errnum;
+        if (m != NULL) {
+            msgid = ldap_msgid(m);
+            msgtype = ldap_msgtype(m);
+            ldap_parse_result(l, m, &errnum, &matched, &error, &refs,
+                              &serverctrls, 1);
+        }
 
-        if (errnum == LDAP_NO_MEMORY)
-            return PyErr_NoMemory();
+        if (msgtype <= 0) {
+            opt_errnum = ldap_get_option(l, LDAP_OPT_ERROR_NUMBER, &errnum);
+            if (opt_errnum != LDAP_OPT_SUCCESS)
+                errnum = opt_errnum;
 
-        if (errnum >= LDAP_ERROR_MIN && errnum <= LDAP_ERROR_MAX)
+            if (errnum == LDAP_NO_MEMORY) {
+                return PyErr_NoMemory();
+            }
+
+            ldap_get_option(l, LDAP_OPT_MATCHED_DN, &matched);
+            ldap_get_option(l, LDAP_OPT_ERROR_STRING, &error);
+        }
+
+        if (errnum >= LDAP_ERROR_MIN && errnum <= LDAP_ERROR_MAX &&
+                errobjects[errnum + LDAP_ERROR_OFFSET] != NULL) {
             errobj = errobjects[errnum + LDAP_ERROR_OFFSET];
-        else
+        }
+        else {
             errobj = LDAPexception_class;
+        }
 
         info = PyDict_New();
-        if (info == NULL)
+        if (info == NULL) {
+            ldap_memfree(matched);
+            ldap_memfree(error);
+            ldap_memvfree((void **)refs);
+            ldap_controls_free(serverctrls);
             return NULL;
+        }
+
+        if (msgtype > 0) {
+            pyresult = PyInt_FromLong(msgtype);
+            if (pyresult)
+                PyDict_SetItemString(info, "msgtype", pyresult);
+            Py_XDECREF(pyresult);
+        }
+
+        if (msgid >= 0) {
+            pyresult = PyInt_FromLong(msgid);
+            if (pyresult)
+                PyDict_SetItemString(info, "msgid", pyresult);
+            Py_XDECREF(pyresult);
+        }
+
+        pyresult = PyInt_FromLong(errnum);
+        if (pyresult)
+            PyDict_SetItemString(info, "result", pyresult);
+        Py_XDECREF(pyresult);
 
         str = PyUnicode_FromString(ldap_err2string(errnum));
         if (str)
@@ -93,8 +137,21 @@ LDAPerror(LDAP *l, char *msg)
             Py_XDECREF(pyerrno);
         }
 
-        if (ldap_get_option(l, LDAP_OPT_MATCHED_DN, &matched) >= 0
-            && matched != NULL) {
+        if (!(pyctrls = LDAPControls_to_List(serverctrls))) {
+            int err = LDAP_NO_MEMORY;
+
+            ldap_set_option(l, LDAP_OPT_ERROR_NUMBER, &err);
+            ldap_memfree(matched);
+            ldap_memfree(error);
+            ldap_memvfree((void **)refs);
+            ldap_controls_free(serverctrls);
+            return PyErr_NoMemory();
+        }
+        ldap_controls_free(serverctrls);
+        PyDict_SetItemString(info, "ctrls", pyctrls);
+        Py_XDECREF(pyctrls);
+
+        if (matched != NULL) {
             if (*matched != '\0') {
                 str = PyUnicode_FromString(matched);
                 if (str)
@@ -104,33 +161,42 @@ LDAPerror(LDAP *l, char *msg)
             ldap_memfree(matched);
         }
 
-        if (errnum == LDAP_REFERRAL) {
-            str = PyUnicode_FromString(msg);
+        if (errnum == LDAP_REFERRAL && refs != NULL && refs[0] != NULL) {
+            /* Keep old behaviour, overshadow error message */
+            char err[1024];
+
+            snprintf(err, sizeof(err), "Referral:\n%s", refs[0]);
+            str = PyUnicode_FromString(err);
+            PyDict_SetItemString(info, "info", str);
+            Py_XDECREF(str);
+        }
+        else if (error != NULL && *error != '\0') {
+            str = PyUnicode_FromString(error);
             if (str)
                 PyDict_SetItemString(info, "info", str);
             Py_XDECREF(str);
         }
-        else if (ldap_get_option(l, LDAP_OPT_ERROR_STRING, &error) >= 0) {
-            if (error != NULL && *error != '\0') {
-                str = PyUnicode_FromString(error);
-                if (str)
-                    PyDict_SetItemString(info, "info", str);
-                Py_XDECREF(str);
-            }
-            ldap_memfree(error);
-        }
+
         PyErr_SetObject(errobj, info);
         Py_DECREF(info);
+        ldap_memvfree((void **)refs);
+        ldap_memfree(error);
         return NULL;
     }
 }
 
+PyObject *
+LDAPerror(LDAP *l)
+{
+    return LDAPraise_for_message(l, NULL);
+}
+
 /* initialise the module constants */
 
 int
 LDAPinit_constants(PyObject *m)
 {
-    PyObject *exc;
+    PyObject *exc, *nobj;
 
     /* simple constants */
 
@@ -160,6 +226,10 @@ LDAPinit_constants(PyObject *m)
 #define add_err(n) do {  \
     exc = PyErr_NewException("ldap." #n, LDAPexception_class, NULL);  \
     if (exc == NULL) return -1;  \
+    nobj = PyLong_FromLong(LDAP_##n); \
+    if (nobj == NULL) return -1; \
+    if (PyObject_SetAttrString(exc, "errnum", nobj) != 0) return -1; \
+    Py_DECREF(nobj); \
     errobjects[LDAP_##n+LDAP_ERROR_OFFSET] = exc;  \
     if (PyModule_AddObject(m, #n, exc) != 0) return -1;  \
     Py_INCREF(exc);  \
