@@ -1,12 +1,8 @@
-# -*- coding: utf-8 -*-
 """
 Automatic tests for python-ldap's C wrapper module _ldap
 
 See https://www.python-ldap.org/ for details.
 """
-
-from __future__ import unicode_literals
-
 import contextlib
 import errno
 import os
@@ -18,7 +14,7 @@ os.environ['LDAPNOINIT'] = '1'
 
 # import the plain C wrapper module
 import _ldap
-from slapdtest import SlapdTestCase, requires_tls
+from slapdtest import SlapdTestCase, requires_tls, requires_init_fd
 
 
 class TestLdapCExtension(SlapdTestCase):
@@ -31,7 +27,7 @@ class TestLdapCExtension(SlapdTestCase):
 
     @classmethod
     def setUpClass(cls):
-        super(TestLdapCExtension, cls).setUpClass()
+        super().setUpClass()
         # add two initial objects after server was started and is still empty
         suffix_dc = cls.server.suffix.split(',')[0][3:]
         cls.server._log.debug(
@@ -55,14 +51,14 @@ class TestLdapCExtension(SlapdTestCase):
         )
 
     def setUp(self):
-        super(TestLdapCExtension, self).setUp()
+        super().setUp()
         self._writesuffix = None
 
     def tearDown(self):
         # cleanup test subtree
         if self._writesuffix is not None:
             self.server.ldapdelete(self._writesuffix, recursive=True)
-        super(TestLdapCExtension, self).tearDown()
+        super().tearDown()
 
     @property
     def writesuffix(self):
@@ -251,12 +247,14 @@ class TestLdapCExtension(SlapdTestCase):
         with self._open_conn_fd() as (sock, l):
             self.assertEqual(l.whoami_s(), "dn:" + self.server.root_dn)
 
+    @requires_init_fd()
     def test_simple_bind_fileno_invalid(self):
         with open(os.devnull) as f:
             l = _ldap.initialize_fd(f.fileno(), self.server.ldap_uri)
             with self.assertRaises(_ldap.SERVER_DOWN):
                 self._bind_conn(l)
 
+    @requires_init_fd()
     def test_simple_bind_fileno_closed(self):
         with self._open_conn_fd() as (sock, l):
             self.assertEqual(l.whoami_s(), "dn:" + self.server.root_dn)
@@ -264,6 +262,7 @@ class TestLdapCExtension(SlapdTestCase):
             with self.assertRaises(_ldap.SERVER_DOWN):
                 l.whoami_s()
 
+    @requires_init_fd()
     def test_simple_bind_fileno_rebind(self):
         with self._open_conn_fd() as (sock, l):
             self.assertEqual(l.whoami_s(), "dn:" + self.server.root_dn)
@@ -288,7 +287,7 @@ class TestLdapCExtension(SlapdTestCase):
             '',
             _ldap.SCOPE_BASE,
             '(objectClass=*)',
-            [str('objectClass'), str('namingContexts')],
+            ['objectClass', 'namingContexts'],
         )
         self.assertEqual(type(m), type(0))
         result, pmsg, msgid, ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
@@ -933,6 +932,29 @@ class TestLdapCExtension(SlapdTestCase):
         package = _ldap.get_option(_ldap.OPT_X_TLS_PACKAGE)
         self.assertIn(package, {"GnuTLS", "MozNSS", "OpenSSL"})
 
+    @unittest.skipUnless(
+        hasattr(_ldap, "OPT_X_TLS_REQUIRE_SAN"),
+        reason="Test requires OPT_X_TLS_REQUIRE_SAN"
+    )
+    def test_require_san(self):
+        l = self._open_conn(bind=False)
+        value = l.get_option(_ldap.OPT_X_TLS_REQUIRE_SAN)
+        self.assertIn(
+            value,
+            {
+                _ldap.OPT_X_TLS_NEVER,
+                _ldap.OPT_X_TLS_ALLOW,
+                _ldap.OPT_X_TLS_TRY,
+                _ldap.OPT_X_TLS_DEMAND,
+                _ldap.OPT_X_TLS_HARD,
+            }
+        )
+        l.set_option(_ldap.OPT_X_TLS_REQUIRE_SAN, _ldap.OPT_X_TLS_TRY)
+        self.assertEqual(
+            l.get_option(_ldap.OPT_X_TLS_REQUIRE_SAN),
+            _ldap.OPT_X_TLS_TRY
+        )
+
 
 if __name__ == '__main__':
     unittest.main()
